"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var Either = require("./Data.Either")
var Main = require("./Main")
var Options = require("./Options")
var purescript_1 = require("./purescript");
var Glue_1 = require("./Glue");
var Source_1 = require("./Source");
var Inference_1 = require("./Inference");
var CombineClasses_1 = require("./CombineClasses");
var TargetLanguage = /** @class */ (function () {
    function TargetLanguage(displayName, names, extension, optionDefinitions) {
        this.displayName = displayName;
        this.names = names;
        this.extension = extension;
        this.optionDefinitions = optionDefinitions;
    }
    return TargetLanguage;
}());
exports.TargetLanguage = TargetLanguage;
var TypeScriptTargetLanguage = /** @class */ (function (_super) {
    __extends(TypeScriptTargetLanguage, _super);
    function TypeScriptTargetLanguage() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TypeScriptTargetLanguage.prototype.transformAndRenderConfig = function (config) {
        var graph;
        if (config.isInputJSONSchema) {
            var glueGraphOrError = Main.glueGraphFromJsonConfig(config);
            if (Either.isLeft(glueGraphOrError)) {
                throw "Error processing JSON: " + purescript_1.fromLeft(glueGraphOrError);
            }
            if (!config.doRender) {
                return { lines: ["Done.", ""], annotations: immutable_1.List() };
            }
            var glueGraph = purescript_1.fromRight(glueGraphOrError);
            graph = Glue_1.glueGraphToNative(glueGraph);
        }
        else {
            var inference_1 = new Inference_1.TypeInference(config.inferMaps, this.supportsEnums && config.inferEnums);
            graph = immutable_1.Map(config.topLevels.map(function (tlc) {
                return [
                    tlc.name,
                    inference_1.inferType(config.compressedJSON, tlc.name, tlc.samples)
                ];
            }));
            if (config.combineClasses) {
                graph = CombineClasses_1.combineClasses(graph);
            }
        }
        if (!config.doRender) {
            return { lines: ["Done.", ""], annotations: immutable_1.List() };
        }
        var renderResult = this.renderGraph(graph, config.rendererOptions);
        return Source_1.serializeRenderResult(renderResult, this.indentation);
    };
    TypeScriptTargetLanguage.prototype.needsCompressedJSONInput = function (rendererOptions) {
        return true;
    };
    Object.defineProperty(TypeScriptTargetLanguage.prototype, "indentation", {
        get: function () {
            return "    ";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TypeScriptTargetLanguage.prototype, "supportsEnums", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return TypeScriptTargetLanguage;
}(TargetLanguage));
exports.TypeScriptTargetLanguage = TypeScriptTargetLanguage;
function optionSpecificationToOptionDefinition(spec) {
    var valueType = Options.valueType(spec.default);
    var type = valueType === "BooleanValue" ? Boolean : String;
    var legalValues;
    var defaultValue = spec.default.value0;
    if (valueType === "EnumValue") {
        var value1 = spec.default.value1;
        legalValues = value1;
        defaultValue = legalValues[defaultValue];
    }
    return {
        name: spec.name,
        description: spec.description,
        typeLabel: spec.typeLabel,
        renderer: true,
        type: type,
        legalValues: legalValues,
        defaultValue: defaultValue
    };
}
function optionDefinitionsForRenderer(renderer) {
    return renderer.options.map(optionSpecificationToOptionDefinition);
}
var PureScriptTargetLanguage = /** @class */ (function (_super) {
    __extends(PureScriptTargetLanguage, _super);
    function PureScriptTargetLanguage(renderer) {
        var _this = this;
        var optionDefinitions = optionDefinitionsForRenderer(renderer);
        _this = _super.call(this, renderer.displayName, renderer.names, renderer.extension, optionDefinitions) || this;
        return _this;
    }
    PureScriptTargetLanguage.prototype.transformAndRenderConfig = function (config) {
        // The PureScript rendering pipeline expects option values to be strings.
        // TargetLangauges expect options to be strings or booleans
        // So we stringify the booleans.
        var options = config.rendererOptions;
        config.rendererOptions = {};
        for (var _i = 0, _a = Object.keys(options); _i < _a.length; _i++) {
            var key = _a[_i];
            config.rendererOptions[key] = options[key].toString();
        }
        config.language = this.names[0];
        var resultOrError = Main.main(config);
        if (Either.isLeft(resultOrError)) {
            throw "Error: " + purescript_1.fromLeft(resultOrError);
        }
        var source = purescript_1.fromRight(resultOrError);
        return { lines: source.split("\n"), annotations: immutable_1.List() };
    };
    PureScriptTargetLanguage.prototype.needsCompressedJSONInput = function (rendererOptions) {
        return false;
    };
    return PureScriptTargetLanguage;
}(TargetLanguage));
exports.PureScriptTargetLanguage = PureScriptTargetLanguage;
