"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var Type_1 = require("./Type");
var TypeBuilder = /** @class */ (function () {
    function TypeBuilder() {
        var _this = this;
        // FIXME: make mutable?
        this._primitiveTypes = immutable_1.Map();
        this._mapTypes = immutable_1.Map();
        this._arrayTypes = immutable_1.Map();
        this._enumTypes = immutable_1.Map();
        this._classTypes = immutable_1.Map();
        this._unionTypes = immutable_1.Map();
        this.getPrimitiveType = function (kind) {
            var t = _this._primitiveTypes.get(kind);
            if (t === undefined) {
                t = new Type_1.PrimitiveType(kind);
                _this._primitiveTypes = _this._primitiveTypes.set(kind, t);
            }
            return t;
        };
        this.getEnumType = function (name, cases) {
            var t = _this._enumTypes.get(cases);
            if (t === undefined) {
                t = new Type_1.EnumType(name, cases);
                _this._enumTypes = _this._enumTypes.set(cases, t);
            }
            else {
                t.addName(name);
            }
            return t;
        };
        this.getMapType = function (values) {
            var t = _this._mapTypes.get(values);
            if (t === undefined) {
                t = new Type_1.MapType(values);
                _this._mapTypes = _this._mapTypes.set(values, t);
            }
            return t;
        };
        this.getArrayType = function (items) {
            var t = _this._arrayTypes.get(items);
            if (t === undefined) {
                t = new Type_1.ArrayType(items);
                _this._arrayTypes = _this._arrayTypes.set(items, t);
            }
            return t;
        };
        this.getClassType = function (name, properties) {
            var t = _this._classTypes.get(properties);
            if (t === undefined) {
                t = new Type_1.ClassType(name, properties);
                _this._classTypes = _this._classTypes.set(properties, t);
            }
            else {
                t.addName(name);
            }
            return t;
        };
        this.getUnionType = function (name, members) {
            var t = _this._unionTypes.get(members);
            if (t === undefined) {
                t = new Type_1.UnionType(name, members);
                _this._unionTypes = _this._unionTypes.set(members, t);
            }
            else {
                t.addName(name);
            }
            return t;
        };
    }
    return TypeBuilder;
}());
exports.TypeBuilder = TypeBuilder;
